/*-----------------------------------------------------------------------*/ 
/*    audio_io.c  Ville Hautamki <villeh@cs.joensuu.fi>, 2001           */ 
/*    wav_io.cpp  Teemu Kilpelinen <tkilpela@cs.joensuu.fi>, 2000-2001  */
/*                 [Don Cross] 16.2.1993,                                */
/*								         */
/*	This module includes functions for audio file loading.	         */
/*                                                                       */
/*	Version Number: 0.02                                             */
/* char *id = "Tue Nov 27 18:23:41 EET 2001";                            */
/*-----------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <sndfile.h>
#include "signal.h"
#include "util.h"
#include "audio_io.h"

/*
  Functioning:  Read voice signal to buffer
	
  Inputs:		char* filename	

  History:		Ville Hautamki 27.11.2001
                        Teemu Kilpelinen and Ismo Krkkinen 15.04.2001
  Version:		1.1 Total rewrite to use libsndfile instead of 
                            Don Cross's C++ shi'it. B-) (Ville)
                        1.0 (original)
  
  Reference:		
*/

Signal *ReadSignal(char *filename)
{
  int i = 0, readcount, start = 0, count, k;
  SF_INFO sfinfo;
  SNDFILE *infile;
  Signal *p; // Pointer to signal structure 
  double buffer[BUFFER_LEN];
    
  infile = sf_open_read(filename, &sfinfo);
  if (!infile) { 
    printf("Could not open file %s\n", filename);
    exit(-1);
  }

  p = create_signal(sfinfo.samples);
  set_srate(p, sfinfo.samplerate);
  set_bps(p, sfinfo.pcmbitwidth);
  set_nch(p, sfinfo.channels);
  count = sfinfo.samples;  

  while ((readcount = sf_read_double(infile, buffer, BUFFER_LEN, 1))) { 
    if (start > readcount) { 
      start -= readcount ;
      continue ;
    }
        
    for (k = start; k < readcount && count > 0; k++, count --) {
      put_sample(p,i, buffer[k]);
      i++;
    }

    if (k == count)
      break;
  }

  sf_close(infile);

  return p;
} 
