/* identifyuser.c by Ville Hautamki <villeh@cs.joensuu.fi>, 2002
   All sources are under GPL */

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h> // Needed for gettimeofday()
#include <time.h>
#include <unistd.h>   // Needed for getopt() 
#include <string.h>

#include "util.h"
#include "speakerid.h"
#include "cmdline.h"

char *id = "identifyuser: Mon Jun 10 17:13:31 EET DST 2002";

long setwatch();
void prtime(long start);
Parameters *init_params(void);
Personal *init_personal(void);
//void parse_params(Parameters *parap, int argc, char **argv);
void print_parameters(Parameters *parap);
void print_usage(); 
void print_results(Nbest *results);
void free_parameters(Parameters *parap);
void cp_params(Parameters *parap, Personal *perptr,
	       struct gengetopt_args_info *args_info);
void find_lowest_score(Parameters *parap, Nbest *results, int id);

int main(int argc, char **argv)
{
  long watch;
  Parameters *parap;
  Personal *perptr;
  Nbest *results, *result_head;
  struct gengetopt_args_info args_info;
  
  if (cmdline_parser(argc, argv, &args_info) != 0)
    exit(1) ;

  watch = setwatch();
  parap = init_params();
  perptr = init_personal();

  cp_params(parap, perptr, &args_info);
  print_parameters(parap);
 
  if ((results = find_most_similar_spkr(parap)) == NULL) {
    printf("Error in find_most_similar_spkr():speaker model does not exist\n");
    exit(-1);
  }
    
  result_head = results;
  find_lowest_score(parap, results, perptr->id);
  //  print_results(results);
  prtime(watch);

  remove_nbest(result_head);
  free_parameters(parap);

  return 0;
}

void find_lowest_score(Parameters *parap, Nbest *results, int id)
{
  double lowest = 10000000.0;
  Nbest *lowele = NULL;

  for (;results; results = results->next) {
    if (results->match_score < lowest) {
      lowest = results->match_score;
      lowele = results;
    }
  }

  if (lowele->id == id) 
    printf("YES modelsize: %d id: %d\n", parap->modelptr->model_size, id);
  else 
    printf("NO modelsize: %d id: %d\n", parap->modelptr->model_size, id);
}

void print_results(Nbest *results)
{
  for (;results; results = results->next) 
    printf("SpeakerID: %d Match_Score: %f Confidence: %f\n", results->id, 
	   results->match_score, results->confidence);
}

long setwatch()
{
  time_t tmp;
  return (long) time(&tmp);
}

void prtime(long start)
{
  struct timeval tv;
  double elapsed;

  gettimeofday(&tv, NULL);
  elapsed = (double)tv.tv_sec + ((double)tv.tv_usec /1e6) - start;

  printf("Time: %6.1f\n", elapsed);
}

Parameters *init_params(void)
{
  Parameters *parap;

  parap = (Parameters *) allocate(sizeof(Parameters), NO_INIT);

  parap->instr = NULL;
  parap->speaker_db = NULL;
  parap->path_models = NULL;
  parap->featptr = NULL;
  parap->modelptr = NULL;
  parap->matchptr = NULL;
  parap->deciptr = NULL;

  return parap;
}

void parse_params(Parameters *parap, Personal *perptr, int argc, char **argv)
{
  int c;

  printf("Parsing parameters\n");

  while (1) {
    c = getopt(argc, argv, "i:s:h:p:");
    if (c == -1)
      return;

    switch (c) {
    case 'i':
      parap->instr = (char *) allocate(sizeof(char)*strlen(optarg), NO_INIT);
      parap->instr = strcpy(parap->instr, optarg);
      break;
    case 's':
      parap->speaker_db = (char *) allocate(sizeof(char)*strlen(optarg), 
					    NO_INIT);
      parap->speaker_db = strcpy(parap->speaker_db, optarg);
      break;
    case 'p':
      parap->path_models = (char *) allocate(sizeof(char)*strlen(optarg), 
					    NO_INIT);
      parap->path_models = strcpy(parap->path_models, optarg);
      break;
    case 'h':
      print_usage();
      break;
    default:
      print_usage();
    }
  }
}

void print_parameters(Parameters *parap)
{ 
  printf("%s\n", id);
  printf("By Ville Hautamki, Teemu Kilpelinen, Tomi Kinnunen, Ismo Krkkinen\n");
  printf("Inputfile: %s\n", parap->instr);
  printf("SpeakerDB: %s\n", parap->speaker_db); 
  // Global parameters for whole signal
  printf("Window len: %d\n", parap->featptr->win_length);
  printf("Window shift: %d\n", parap->featptr->win_shift_ms);
  printf("Remove sil: %d\n", parap->featptr->remove_sil);
  printf("Remove DC: %d\n", parap->featptr->remove_dc_offset);
  printf("Window function: %d\n", parap->featptr->win_function_type);
  printf("High emph coeff: %f\n", parap->featptr->high_emph_coeff);
  // Cepstral coeffient parameters
  printf("Feature type: %d\n", parap->featptr->feat_type);
  printf("Num. Mel filters: %d\n", parap->featptr->num_mel_filters);
  printf("Include delta: %d\n", parap->featptr->include_delta);
  printf("Include delta delta: %d\n", parap->featptr->include_delta_delta);
  printf("Include 0th coeff: %d\n", parap->featptr->include_0th_coeff);
  printf("MFCC ord: %d\n", parap->featptr->MFCC_ord);
  printf("Lifter type: %d\n", parap->featptr->lifter_type);
  printf("Absolute magnitude: %d\n", parap->featptr->absolute_magnitude);
  //Modeling parameters
  printf("Model size: %d\n", parap->modelptr->model_size);
  printf("Max iterations: %d\n", parap->modelptr->max_iterations);
  printf("Stop threshold: %f\n", parap->modelptr->stop_threshold);
  printf("Num. initial sol.: %d\n", parap->modelptr->num_initial_solutions);
  //Matching parameters
  printf("Matching func: %d\n", parap->matchptr->matching_function);
  printf("Using CB in matching: %d\n", parap->matchptr->using_cb);
  printf("Max iterations in CB matching: %d\n", parap->matchptr->maxiter);
  //Decision parameters
  printf("Decision func: %d\n", parap->deciptr->decision_function);
  printf("Number of best matches: %d\n", parap->deciptr->num_best);
  printf("Confidence measure: %d\n", parap->deciptr->confidence_measure);
}


void print_usage()
{
  printf("%s\n", id);
  printf("By Ville Hautamki, Teemu Kilpelinen, Tomi Kinnunen, Ismo Krkkinen\n");
  printf("usage: identifyuser [options]\n");
  printf("-i <input file>\n");
  printf("-s <speaker db>\n");
  printf("-p <path to models>\n");
  printf("-h   Display this information\n");

  exit(-1);
}


Personal *init_personal(void)
{
  Personal *perptr;

  perptr = (Personal *) allocate(sizeof(Personal), NO_INIT);

  perptr->id = -1;       //-1 == NULL in this case
  perptr->name = NULL;
  perptr->gender = -1;   // -1 == NULL in this case
  perptr->age = -1;      // -1 == NULL in this case
  perptr->dialect = -1;  // -1 == NULL in this case

  return perptr;
}


void free_parameters(Parameters *parap)
{
  free(parap->deciptr);
  free(parap->matchptr);
  free(parap->modelptr);
  free(parap->featptr);
  if (parap->path_models != NULL)
    free(parap->path_models);
  free(parap->speaker_db);
  free(parap->instr);
  free(parap);
}

void cp_params(Parameters *parap, Personal *perptr, 
	       struct gengetopt_args_info *args_info)
{

  parap->featptr = (Featextract *) allocate(sizeof(Featextract),NO_INIT);
  parap->modelptr = (Modeling *) allocate(sizeof(Modeling),NO_INIT);
  parap->matchptr = (Matching *) allocate(sizeof(Matching),NO_INIT);
  parap->deciptr = (Decisionlogic *) allocate(sizeof(Decisionlogic),NO_INIT);

  //Basic parameters
  parap->instr = args_info->input_arg;
  parap->speaker_db = args_info->speakerdb_arg;
  parap->path_models = args_info->modelpath_arg;
  // Global parameters for whole signal
  parap->featptr->win_length = args_info->win_length_arg;
  parap->featptr->win_shift_ms = args_info->win_shift_ms_arg;
  parap->featptr->remove_sil = args_info->remove_sil_flag;
  parap->featptr->sil_thr = args_info->sil_thr_arg;
  parap->featptr->remove_dc_offset = args_info->remove_dc_offset_flag;
  parap->featptr->win_function_type = args_info->win_function_type_arg;
  parap->featptr->high_emph_coeff = args_info->high_emph_coeff_arg;
  // Cepstral coeffient parameters
  parap->featptr->feat_type = args_info->feat_type_arg;
  parap->featptr->num_mel_filters = args_info->num_mel_filters_arg;
  parap->featptr->include_delta = args_info->include_delta_flag;
  parap->featptr->include_delta_delta = args_info->include_delta_delta_flag;
  parap->featptr->include_0th_coeff = args_info->include_0th_coeff_flag;
  parap->featptr->MFCC_ord = args_info->MFCC_ord_arg;
  parap->featptr->lifter_type = args_info->lifter_type_arg;
  parap->featptr->absolute_magnitude = args_info->absolute_magnitude_flag;
  //Modeling parameters
  parap->modelptr->model_size = args_info->model_size_arg;
  parap->modelptr->max_iterations = args_info->max_iterations_arg;
  parap->modelptr->stop_threshold = args_info->stop_threshold_arg;
  parap->modelptr->num_initial_solutions = args_info->initial_solutions_arg; 
  //Matching parameters
  parap->matchptr->matching_function = args_info->matching_function_arg;
  parap->matchptr->using_cb = args_info->using_cb_flag;
  parap->matchptr->maxiter = args_info->maxiter_arg;
  //Decision parameters
  parap->deciptr->decision_function = args_info->decision_function_arg;
  parap->deciptr->num_best = args_info->nbest_arg;
  parap->deciptr->confidence_measure = args_info->confidence_measure_arg;
  //Personal parameters
  perptr->id = args_info->id_arg;
  perptr->name = args_info->name_arg;
  perptr->gender = args_info->gender_arg;
  perptr->age = args_info->age_arg;
  perptr->dialect = args_info->dialect_arg;
}
