/* csv.c - CSV parsing routines by Ville Hautamki <villeh@cs.joensuu.fi>, 2002
   All sources are under GPL */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "list.h"
#include "util.h"
#include "csv.h"

/* High level CSV parsing routine.
 * Takes as parameters pointer to list head and pointer to CSV file.
 * Returns 0 on success.
 */

int csv_parse(List *listp, FILE *infile)
{
  int flag = 0; 
  Csv *csvptr = NULL;
  Element *elep = NULL;


  csvptr = csv_create(infile, &flag); 
  while (!flag) {

    if (csvptr == NULL) 
      return 0;

    elep = (Element *) allocate(sizeof(Element), NO_INIT);
    elep->path = (char *) allocate(sizeof(char)*BUF, NO_INIT);
    elep->name = (char *) allocate(sizeof(char)*BUF, NO_INIT);

    elep->id = atoi(csv_get_field(csvptr, 0));
    elep->path = strcpy(elep->path, csv_get_field(csvptr, 1));
    elep->name = strcpy(elep->name, csv_get_field(csvptr, 2));
    elep->gender = atoi(csv_get_field(csvptr, 3));
    elep->age = atoi(csv_get_field(csvptr, 4));
    elep->dialect = atoi(csv_get_field(csvptr, 5));

    insert_element(listp, elep);
    csv_destroy(csvptr);
    csvptr = NULL;
    csvptr = csv_create(infile, &flag); 
  }

  return 0;
}

/* Writes CSV file to disk.
 * Takes as parameters pointer to list head and pointer to CSV file.
 * Returns 0 on success.
 */

int csv_write(List *listp, FILE *outfile)
{
  Iterator *itep; 
  Element *elep;

  itep = iterate_start();
  while ((elep = iterate_element(listp, itep)) != NULL) 
    fprintf(outfile, "%d,%s,%s,%d,%d,%d\n", 
	    elep->id, elep->path, elep->name, elep->gender, 
	    elep->age, elep->dialect);

  return 0;
}

/* Reads one line from CSV file.
 * Takes as parameters pointer to CSV file and pointer to buffer, where 
 * line is stored.
 * Returns returns flag 1, when there is still lines to be read 
 * and 0 when EOF is encountered. (This could be nicer...)
 */

int readline(FILE *infile, char *buf)
{
  if ((fgets(buf, BUF, infile)) == NULL) {
    return 1;  
  }

 return feof(infile);
}

/* Parses CVS fields into CVS struct. 
 * Takes as parameters pointer to CSV file and pointer to flag, when 
 * which describes when we are at the end, this flag propagates from 
 * readline().
 * Returns pointer to Cvs struct. Which contains relevant fields and
 * number of fields read.
 */

Csv *csv_create(FILE *infile, int *flag)
{
  int i;
  Csv *csvptr;
  char buf[BUF], *p, *q, *temp;


  csvptr = (Csv *) allocate(sizeof(Csv), NO_INIT);
  
  *flag = readline(infile, buf);

  i = 0;
  for (q = buf; (p = strtok(q, ",\n\r")) != NULL; q = NULL) {
    temp = (char *) allocate(sizeof(char)*FIELDLEN, NO_INIT);
    temp = strncpy(temp, p, FIELDLEN);
    csvptr->fields[i++] = temp; 
  }

  csvptr->num_fields = i;
  
  return csvptr;
}

/* Destroy Cvs struct.
 * Takes as parameter pointer to Cvs struct.
 * Does not return anything.
 */

void csv_destroy(Csv *csvptr)
{
  int i;
  for (i = 0; i < csvptr->num_fields; i++)
    free(csvptr->fields[i]);
  free(csvptr);
}

/* Gets one field from Cvs struct. 
 * Takes as parameters pointer to Cvs struct and field number.
 * Returns pointer to string stored in the field.
 */

char *csv_get_field(Csv *csvptr, int field)
{
  return csvptr->fields[field];
}
