#ifndef FVEC_H
#define FVEC_H

#include <math.h>
#include "memctrl.h"
#include <stdio.h>

#define fvNew(Dimensionality) allocate(sizeof(double) * (Dimensionality), NO_INIT) 
#define fvDelete(V) free(V)

void fvCopy(double* From, double* To, int Dimensionality);
void fvFill(double* V, double Value, int Dimensionality);
void fvRandom(double* V, int Dimensionality);

double fvManhattanDistance(double* V1, double* V2, int Dimensionality);
double fvManhattanLength(double* V, int Dimensionality);

double fvDistance(double* V1, double* V2, int Dimensionality);
#define fvLength(V, D) sqrt(fvScalarProduct((V), (V), (D)))
#define fvNormalize(V, Dimensionality) \
	fvScale((V), 1.0 / fvLength((V), (Dimensionality)))

double fvScalarProduct(double* V1, double* V2, int Dimensionality);
void fvAdd(double* V1, double* V2, double* Result, int Dimensionality);
void fvSubtract(double* V1, double* V2, double* Result, int Dimensionality);
void fvScale(double* V, double Scale, int Dimensionality);

void fvSort(double* V, int Dimensionality, int Ascending);

double** fvNewSet(int Count, int Dimensionality);
void fvDeleteSet(double** Set, int Count);
double** fvCopySet(double** Dest, double** Src, int Count, int Dimensionality);
int fvWriteSet(double** Set, int Count, int Dimensionality, FILE* f);
int fvReadSet(double*** Set, int* Count, int* Dimensionality, FILE* f);

void fvFillSet(double** Set, int Count, int Dimensionality, double Value);

double** fvSubSetDimensions(double** Set, int Count, int Dimensionality,
    int* SelectedDimensions, double** SubSet, int* SubsetDimensionality);
double fvSetAverage(double** Set, int Count, int Dimension);
double fvSetNth(double** Set, int Count, int Dimension, int N);
double fvSetMinimum(double** Set, int Count, int Dimension);
double fvSetMaximum(double** Set, int Count, int Dimension);
double fvSetTotalMinimum(double** Set, int Count, int Dimensionality);
double fvSetTotalMaximum(double** Set, int Count, int Dimensionality);
int fvSetIsBinary(double** Set, int Count, int Dimensionality,
    double V1, double V2);

enum { fvAscendingOrder, fvDescendingOrder };

void fvSortSet(double** Set, int Count, int Dimensionality,
    int* DimensionOrder, int* ValueOrder, int Order);
double* fvSetUnique(double** Set, int Count, int Dimension, int* FoundCount);

#endif
