/* list.c  by Ville Hautamki <villeh@cs.joensuu.fi>, 2001
   All sources are under GPL */

#include <stdio.h>
#include <stdlib.h>

#include "list.h"
#include "util.h"


/* Creates and list head element. 
 * No parameters.
 * Returns pointer to head element.
 */

List *create_list(void)
{
  List *p;

  p = (List *) allocate(sizeof(List), NO_INIT);

  p->top = NULL;
  p->total_elements = 0;

  return p;
}

/* Inserts element to list.
 * Takes as parameters pointer to head element, and pointer to element to be 
 *  added. Element has to be created beforehand.
 * Does not return anything.
 */

void insert_element(List *listp, Element *elep)
{
  if (listp->top == NULL) 
    elep->next = NULL; 
  else {
    elep->next = listp->top;
    listp->top->prev = elep;
  }

  elep->prev = NULL;
  listp->top = elep;
  listp->total_elements++;
}

/* Frees whole list. 
 * Takes as parameter pointer to head element.
 * Does not return anything.
 */

void free_list(List *listp)
{
  while (listp->top != NULL) 
    free_element(listp, listp->top);
}

/* Frees one element in list.
 * Takes as parameters pointer to head element and pointer to element to be 
 *  freed.
 * Does not return anything.
 */

void free_element(List *listp, Element *elep)
{
  if (elep == listp->top) {
    listp->top = elep->next;
    if (listp->top != NULL)  
      listp->top->prev = NULL;
  } else if (elep->next == NULL) {
    elep->prev->next = NULL;
  } else {
    elep->next->prev = elep->prev;
    elep->prev->next = elep->next;
  }
  
  listp->total_elements--;
  free(elep);
}

/* Starts list iterator.
 * Does not take any parameters.
 * Returns pointer to iterator. 
 */

Iterator *iterate_start(void)
{
  Iterator *p;

  p = (Iterator *) allocate(sizeof(Iterator), NO_INIT);

  p->curele = NULL;

  return p;
}

/* Iterates to next element in list.
 * Takes as parameters pointer to list head and pointer to iterator.
 * Returns pointer to next element.
 */

Element *iterate_element(List *listp, Iterator *itep)
{

  if (itep->curele == NULL)
    itep->curele = listp->top;
  else if (itep->curele->next != NULL)
    itep->curele = itep->curele->next;
  else {
    free(itep);
    return NULL;
  }

  return itep->curele;
}

/* Prints list.
 * Takes as parameter pointer to list head.
 * Does not return anything.
 */

void print_list(List *listp)
{
  Iterator *itep;
  Element *elep;

  itep = iterate_start();
  while ((elep = iterate_element(listp, itep)) != NULL) 
    printf("%d %s %s %d %d %d\n", elep->id, elep->path, elep->name,
	   elep->gender, elep->age, elep->dialect);
}

/* Finds element by ID number.
 * Takes as parameters pointer to list head and speaker id as integer.
 * Returns pointer to list element found or when it is not found 
 *  returns NULL.
 */

Element *find_element(List *listp, int id)
{
  Iterator *itep;
  Element *elep;

  itep = iterate_start();
  while ((elep = iterate_element(listp, itep)) != NULL) 
    if (elep->id == id)
      return elep;

  if (elep->id != id)
    return NULL;

  return elep;
}

/* Gets top element in the list.
 * Takes as parameter pointer to list head.
 * Returns pointer to top list element.
 */

Element *get_element(List *listp)
{
  if (listp->top != NULL)
    return listp->top;
  else {
    printf("get_element: Model database is empty!\n");
    exit(-1);
  }     
}
