#ifndef MATRIX_H
#define MATRIX_H

#include <stdio.h>

typedef struct {
   int	   Rows;
   int	   Cols;
   double*  Data;
} MATRIX_TYPE;

typedef enum {
   smooth_Gauss,
   Lp_similarity
} VECTOR_SIMILARITY_TYPE;

typedef MATRIX_TYPE* MATRIX;

#define MRows(M) ((M)->Rows)
#define MCols(M) ((M)->Cols)
#define IsSquare(M) ((M)->Rows == (M)->Cols) 
#define IsScalar(M) (((M)->Rows == 1) && ((M)->Cols == 1))
#define IsColVector(M) ( ((M)->Cols == 1) )
#define IsRowVector(M) ( ((M)->Rows == 1) )

/* Memory control -------------------------------------------------- */
MATRIX	NewMatrix(int Rows, int Cols);
void	DeleteMatrix(MATRIX M);
void 	CopyMatrix(MATRIX From, MATRIX To);

/* Matrix I/O ------------------------------------------------------ */
void    PrintMatrix(MATRIX M, FILE* f);
void	SetMatrixElement(MATRIX M, int Row, int Col, double Value);
double	GetMatrixElement(MATRIX M, int Row, int Col);
MATRIX	GetRow(MATRIX TargetVector, MATRIX M, int Row);
MATRIX  GetColumn(MATRIX TargetVector, MATRIX M, int Col);
MATRIX	SetRow(MATRIX M, MATRIX Data, int Row);
MATRIX  SetColumn(MATRIX M, MATRIX Data, int Col);
MATRIX  InsertNewVectorToMatrix(MATRIX M, MATRIX Vector, int ColVec);
MATRIX	Diag(MATRIX TargetVector, MATRIX M);

/* Matrix operations ----------------------------------------------- */
int	Same(MATRIX A, MATRIX B);
MATRIX	Transpose(MATRIX M);
MATRIX	IdentityMatrix(MATRIX M);
MATRIX	ZeroMatrix(MATRIX M);
MATRIX  OneMatrix(MATRIX M);
MATRIX	RandomMatrix(MATRIX M);
MATRIX	MultiplyByScalar(MATRIX Result, MATRIX M, double Scalar);
MATRIX	MatrixSum(MATRIX Result, MATRIX A, MATRIX B);
MATRIX	MatrixSubstract(MATRIX Result, MATRIX A, MATRIX B);
MATRIX	MatrixProduct(MATRIX Result, MATRIX A, MATRIX B);
double	Determinant(MATRIX M);
double	Trace(MATRIX A);
MATRIX  MatrixInverse(MATRIX Result, MATRIX A);
double   ProdOfElements(MATRIX M);     
double   SumOfElements(MATRIX M);  
double   MaxElem( MATRIX M );
double   MinElem( MATRIX M );

/* Vector operations ----------------------------------------------- */
double 	VectorDistance(MATRIX x, MATRIX y, int p);	
double 	Avg(MATRIX Vector);
double	Var(MATRIX Vector, int UnbiasedEstimate);
double	Norm(MATRIX Vector);
double	InnerProduct(MATRIX Vec1, MATRIX Vec2);
double	VectorSimilarity( MATRIX Vec1, MATRIX Vec2, 
			  VECTOR_SIMILARITY_TYPE SimType,
			  double Alpha );
void	NormalizeSumToUnity(MATRIX x);

#endif
