#ifndef PATTERN_H
#define PATTERN_H

#include "matrix.h"

typedef struct {
   int	   NumberOfVectors;
   int     Dimensionality;
   MATRIX  Data;
} PATTERN_TYPE;
typedef PATTERN_TYPE* PATTERN;

typedef struct {
   int      NumberOfPatterns;
   PATTERN  *Data;
} PATTERN_SET_TYPE;
typedef PATTERN_SET_TYPE* PATTERN_SET;

/* Basic macros -------------------------------------- */
#define PNumVectors(X) 	     ((X)->NumberOfVectors)
#define PDimensionality(X)   ((X)->Dimensionality)
#define PSDimensionality(PS) ((PS)->Dimensionality)
#define PSNumPatterns(PS)    ((PS)->NumberOfPatterns)

/* Memory control ------------------------------------ */
PATTERN     NewPattern(int Dimensionality, int NumVectors);
void 	    DeletePattern(PATTERN X);
PATTERN_SET NewPatternSet();
void	    DeletePatternSet(PATTERN_SET PS);

/* Pattern I/O --------------------------------------- */
void 	PrintPattern(PATTERN X, FILE* f);
void	PrintPatternSet(FILE* f, PATTERN_SET PS);
PATTERN LoadPattern(char* Filename);
MATRIX	GetVector(MATRIX TargetVector, PATTERN X, int Index);
PATTERN SetVector(PATTERN X, MATRIX Vector, int Index);
PATTERN InsertNewVector(PATTERN X, MATRIX Vector);
PATTERN PickRandomVectors(PATTERN Target, PATTERN Source);
void    CopyVectors( PATTERN Target, PATTERN Source,
                     int StartPos, int EndPos);
PATTERN GetPattern(PATTERN_SET PS, int Index);
void    InsertNewPattern(PATTERN P, PATTERN_SET PS);
PATTERN GetPattern(PATTERN_SET PS, int Index);
PATTERN_SET LoadDataSets( char *FileListName );
int     NumVectors(PATTERN P);

/* Statistical operations ---------------------------- */
MATRIX	MeanVector(MATRIX TargetVector, PATTERN X);
MATRIX  CovMatrix(MATRIX TargetMatrix, PATTERN X);
MATRIX  Variance(MATRIX TargetVector, PATTERN X);
MATRIX  StandardDev(MATRIX TargetVector, PATTERN X);
MATRIX	ScatterMatrix(MATRIX TargetMatrix, PATTERN X);
double	ComputeMeanDistVariance(PATTERN X); 
void    ZeroMeanUnitVarianceNormalize(PATTERN X); 
void    NormalizePatternsZmUv(PATTERN_SET PS);
PATTERN TranslatePattern(PATTERN X, MATRIX Vector);

/* Pattern matching stuff ---------------------------- */
int	FindNearestVector(MATRIX Vector, PATTERN P);
int	FindMostSimilarVector(MATRIX Vector, PATTERN P);
double   DistanceToNearestVector(MATRIX Vector, PATTERN P);
double	MeanSquaredError(PATTERN X, PATTERN C);
double   BhattacharyyaDistance_Gauss( MATRIX Cov1,  MATRIX Cov2,
                                     MATRIX Mean1, MATRIX Mean2 );
double DatasetSimilarity( PATTERN X, PATTERN C,     
                         MATRIX  WeightTable ,     
                         int     Fuzzy,     
                         int     Weighted,     
                         double   Alpha );

double MeanWeightedError( PATTERN X, PATTERN C,
			 MATRIX  WeightTable,
			 int     Fuzzy,
			 double	 Alpha );                                

#endif /* PATTERN_H */
