/* removeuser.c by Ville Hautamki <villeh@cs.joensuu.fi>, 2002
   All sources are under GPL */

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h> // Needed for gettimeofday()
#include <time.h>
#include <unistd.h>   // Needed for getopt() 
#include <string.h>

#include "util.h"
#include "speakerid.h"

char *id = "removeuser: Wed May  8 02:46:48 EET DST 2002";

long setwatch();
void prtime(long start);
Parameters *init_params(void);
Personal *init_personal(void);
void parse_params(Parameters *parap, Personal *perptr, int argc, char **argv);
void print_usage(); 

int main(int argc, char **argv)
{
  long watch;
  Parameters *parap;
  Personal *perptr;

  watch = setwatch();
  parap = init_params();
  perptr = init_personal();
  parse_params(parap, perptr, argc, argv); 

  //Check if user specified enough commandline parameters.
  if (parap->speaker_db == NULL)
    print_usage();

  //Values for testing!
  perptr->name = "Foobar";
  perptr->gender = 1;
  perptr->age = 15;
  perptr->dialect = 2;

  if (remove_speaker(parap, perptr->id)){
    printf("remove_speaker: No such speaker!!");
    exit(-1);
  }
    
  prtime(watch);

  free(parap);

  return 0;
}


/* Initializes time (for calculating running time).
 * Does not take any parameters.
 * Returns time (in seconds) See man 2 time
 *
 */

long setwatch()
{
  time_t tmp;
  return (long) time(&tmp);
}

/* Prints running time. 
 * Takes as a parameter initialized time (from setwatch())
 * Does not return anything.
 */

void prtime(long start)
{
  struct timeval tv;
  double elapsed;

  gettimeofday(&tv, NULL);
  elapsed = (double)tv.tv_sec + ((double)tv.tv_usec /1e6) - start;

  printf("Time: %6.1f\n", elapsed);
}

/* Allocates memory and initializes Parameter struct.
 * Does not take any parameters.
 * Returns pointer to Parameters struct.
 */

Parameters *init_params(void)
{
  Parameters *parap;

  parap = (Parameters *) allocate(sizeof(Parameters), NO_INIT);

  parap->instr = NULL;
  parap->speaker_db = NULL;
  parap->path_models = NULL;
  parap->featptr = NULL;
  parap->modelptr = NULL;
  parap->matchptr = NULL;
  parap->deciptr = NULL;

  return parap;
}

/* Parses commandline parameters with getopt(). 
 * Takes as parameters pointers Parameters struct, pointer to Personal struct,
 *  number of commandline parameters and pointer to actual commandline args.
 * Does not return anything.
 */

void parse_params(Parameters *parap, Personal *perptr, int argc, char **argv)
{
  int c;

  printf("Parsing parameters\n");

  while (1) {
    c = getopt(argc, argv, "s:hI:");
    if (c == -1)
      return;

    switch (c) {
    case 's':
      parap->speaker_db = (char *) allocate(sizeof(char)*strlen(optarg), 
					    NO_INIT);
      parap->speaker_db = strcpy(parap->speaker_db, optarg);
      break;
    case 'I':
      perptr->id = atoi(optarg);
      break;
    case 'h':
      print_usage();
      break;
    default:
      print_usage();
    }
  }
}

/* Allocates memory and initializes Personal struct. 
 * Does not take any parameters.
 * Returns pointer to Personal struct.
 */

Personal *init_personal(void)
{
  Personal *perptr;

  perptr = (Personal *) allocate(sizeof(Personal), NO_INIT);

  perptr->id = -1;       //-1 == NULL in this case
  perptr->name = NULL;
  perptr->gender = -1;   // -1 == NULL in this case
  perptr->age = -1;      // -1 == NULL in this case
  perptr->dialect = -1;  // -1 == NULL in this case

  return perptr;
}

/* Prints out how to use this binary.
 * Does not take any parameters.
 * Does not return anything.
 */

void print_usage()
{
  printf("%s\n", id);
  printf("By Ville Hautamki, Teemu Kilpelinen, Tomi Kinnunen, Ismo Krkkinen\n");
  printf("usage: removeuser [options] -s <speaker db>\n");
  printf("-I               Unique userid\n");
  printf("-h               Display this information\n");

  exit(-1);
}

