/* signal.c by Ville Hautamki <villeh@cs.joensuu.fi>, 2002
   All sources are under GPL */

#include <stdio.h>
#include <stdlib.h>

#include "signal.h"
#include "util.h"

/* Creates a signal data structure. 
 * Takes as a parameter number or samples in signal.
 * Returns pointer to signal.
 */

Signal *create_signal(int num_samples)
{
  Signal *p;

  p = (Signal *) allocate(sizeof(Signal), NO_INIT);
  p->signal_buffer = (double *) allocate(sizeof(double)*num_samples, NO_INIT);
  p->num_samples = num_samples;

  return p;
}

/* Frees signal data structure.
 * Takes as a parameter pointer to signal struct.
 * Function does not return anything.
 */

void free_signal(Signal *ptr)
{
  free(ptr->signal_buffer);
  free(ptr);
}

/* Prints signal to file.
 * Takes as a parameter pointer to signal struct and pointer to a FILE.
 * Function does not return anything.
 */

void print_signal(Signal *ptr, FILE *filep)
{
  int i;

  for (i = 0; i < ptr->num_samples; i++) 
    fprintf(filep, "%f ", get_sample(ptr, i));
}

/* Gets one sample from ith postion as double. 
 * Takes as a parameters pointer to signal struct and number of sample to get
 * Returns ith sample as double.
 */

double get_sample(Signal *ptr, int ith_sample) 
{
  return *(ptr->signal_buffer+ith_sample);
}


/* Puts one double typed sample to ith position. 
 * Takes as a parameters pointer to signal struct, number of sample to put 
 *  and sample value.
 * Function does not return anything.
 */

void put_sample(Signal *ptr, int ith_sample, double value) 
{
  *(ptr->signal_buffer+ith_sample) = value;
}

/* Sets signal sample rate parameter. 
 * Takes as a parameters pointer to signal struct and sample rate as int
 * Function does not return anything.
 */

void set_srate(Signal *ptr, int srate)
{
  ptr->srate = srate;
}

/* Gets signal sample rate.
 * Takes as a parameters pointer to signal struct
 * Returns sample rate as int.
 */

int get_srate(Signal *ptr)
{
  return ptr->srate;
}

/* Sets signal bits per sample rate.
 * Takes as a parameters pointer to signal struct and bits per sample rate.
 * Function does not return anything.
 */

void set_bps(Signal *ptr, int bps)
{
  ptr->bps = bps;
}

/* Gets signal bits per sample rate.
 * Takes as a parameters pointer to signal struct.
 * Returns bits per sample rate as int.
 */

int get_bps(Signal *ptr)
{
  return ptr->bps;
}

/* Gets number of samples in signal 
 * Takes as a parameters pointer to signal struct.
 * Returns number of samples.
 */

int get_nsamples(Signal *ptr)
{
  return ptr->num_samples;
}

/* Sets number of channels in signal 
 * Takes as a parameters pointer to signal struct  and number of channels.
 * Function does not return anything.
 */

void set_nch(Signal *ptr, int num_ch)
{
  ptr->num_ch = num_ch;
}
