/* This file includes public functions to speaker recognition module. 
   And needed parameters. 
   Ville Hautamki, <villeh@cs.joensuu.fi>, 2002                
 */
#ifndef SPEAKERID_H
#define SPEAKERID_H

typedef struct Parameters Parameters;
typedef struct Featextract Featextract;
typedef struct Modeling Modeling;
typedef struct Matching Matching;
typedef struct Decisionlogic Decisionlogic;
typedef struct Personal Personal;
typedef struct Nbest Nbest;

struct Parameters {
  char *instr;                 //path to input file
  char *speaker_db;            //path to speaker database(which is a flat file)
  char *path_models;           //path to models, do not forget trailing /
  char *featurefile;           //path to feature file
  Featextract *featptr;        // If NULL, then set factory defaults
  Modeling *modelptr;          // If NULL, then set factory defaults
  Matching *matchptr;          // If NULL, then set factory defaults
  Decisionlogic *deciptr;      // If NULL, then set factory defaults
};

// This version uses GLA for modeling
struct Modeling {               
  int model_size;              //How many clusters per speaker?
  int max_iterations;          //Stoping criterion 
  double stop_threshold;       //Other stopping crit. 
  int num_initial_solutions;   //Number of initial solutions, for non-det. GLA
};

enum {
  AVDIST = 1
}; 

struct Matching {
  int matching_function;       //Enumerated type: 
                               //  AVDIST = 1;
  int using_cb;                //Create codebook also for matching
  int maxiter;                 //Max iterations for codebook generation def. 10
  int cb_size;                 //Intermediate codebook size def 0, then 
};                             // 1/10 num. vectors in Cepstrum

enum {
  NBEST = 1
}; 

enum {
  NNRATIO = 1
}; 

struct Decisionlogic {
  int decision_function;        //Enumerated type: 
                                //  NBEST = 1;
  int num_best;                 //If NBEST, then what is n?
  int confidence_measure;       //Enumerated list NNRATIO
};

enum {
  HAMMING = 1,
  BARLETT = 2, 
  HANNING = 3, 
  BLACKMAN = 4,
  GAUSSIAN = 5
};

enum {
  MAGNITUDESPECTR = 1,
  POWERSPECTR = 0
};

struct Featextract {
  // Global parameters for whole signal
  int win_length;              //window length in ms
  int win_shift_ms;            //window shift in ms
  int remove_sil;              //remove silence, not implemented yet
  double sil_thr;                 //Threshold for silence removal 
  int remove_dc_offset;        //remove DC-offcet
  int win_function_type;       /* window function type 
                                  1 = HAMMING
                                  2 = RECTANGULAR
                                  3 = BARLETT
                                  4 = BLACKMAN */
  double high_emph_coeff;       //high emphasis coefficients
  // Cepstral coeffient parameters
  int feat_type;               //feature-type: MFCC, ..., not implemented yet 
  int num_mel_filters;         // number of mel-filters
  int include_delta;              // not implemented yet
  int include_delta_delta;        // not implemented yet
  int include_0th_coeff;       //include 0th coefficient 
  int MFCC_ord;                //MFCC-order =~ dimension of result vector
  int lifter_type;             /* lifterfunction type 
                                  1 = truncated
                                  2 = raised sine
                                  3 = half raised sine */
  int absolute_magnitude;      //absolute magnitude
};

// All information concerning speakers. What fields are needed??
struct Personal {
  int id;                      // Unique spkr ID
  char *name;                  // Name as string
  int gender;                  // male = 0, female = 1
  int age; 
  int dialect;                 // dialect by region id number??? Is this sane
};

// Linked list returned by find_most_similar_spkr()
struct Nbest {
  double match_score;          //Raw values returned by matching functions
  double confidence;           //Confidence value between [0...1]
  int id;                     //id of speaker, when -1 spkr not identified
  Nbest *next;
};

int create_speakerdb(Parameters *paraptr);
int add_speaker(Parameters *paraptr, Personal *perptr); 
int remove_speaker(Parameters *paraptr, int id);
Nbest *find_most_similar_spkr(Parameters *paraptr);
void remove_nbest(Nbest *nbestptr);    //Remove garbage from memory
void print_nbest(Nbest *listhead);
int print_features(Parameters *paraptr);
char *get_version(void);


#endif
