/* util.h by Ville Hautamki <villeh@cs.joensuu.fi>, 2001
   All sources are under GPL */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#include "util.h"

/* malloc() wrapper (if NULL exits and flag selects either malloc() or calloc()
 * Takes as parameters size_t size and flag. NO_INIT, is malloc() and
 *  INIT is calloc().
 * Returns pointer to allocated memory.
 */

void *allocate(size_t size, int flag)
{
  void *temp;

  if (flag == NO_INIT) {
    temp = malloc(size);
    if (temp == NULL) {
      printf("Allocation of %d bytes failed\n", size);
      exit(-1);
    }
  } else {
    temp = calloc(size, sizeof(int));
    if (temp == NULL) {
      printf("Allocation of %d bytes failed\n", size);
      exit(-1);
    }
  }

  return temp;
}

/*  
        Functioning:    Changes ms to samples.
        Inputs:                 int ms                 Sample's length in milliseconds
                                        int sampling_rate  Sampling rate of input signal
        Outputs:                -
        Return value:   Sample's lenght in samples  
        History:                Teemu Kilpelinen and Tomi Kinnunen      15.10.2000
        Version:                1.0 (original)

        Reference:              -

*/        

int Ms2Samples(int ms, int sampling_rate)
{
  return (int)((ms/1000.0) * sampling_rate);
}

/*      int     NearestPowerOfTwo(int frame_length, int round_downwards)
        
        Functioning:    Returns either nearest bigger or nearest smaller 
                                        power of two of frame_length. If round_downwards == 1,
                                        returns the nearest power of two <= frame_length. 
                                        Otherwise returns nearest power of two >= frame_length.  
        
        Inputs:                 int frame_length         Frame length in samples.
                                        int round_downwards  Boolean variable (1 or 0).
        Outputs:                -
        Return value:   Nearest either bigger or smaller power of two of frame_length  
        History:                Teemu Kilpelinen ja Ismo Krkkinen 20.12.2000         Version:                1.0 (original)
        Reference:              -
*/

int NearestPowerOfTwo(int frame_length)
{
  int power = 1;
  while (frame_length > power) power *= 2;

  return power;
} 

/* Print error string. Exits after printing error. 
 * Takes as parameters pointer to error string. 
 * Does not return anything (surprise!)
 */

void print_error(char *errorstr)
{
  printf("Unrecoverable error in speakerid library\n");
  printf("%s\n", errorstr);
  exit(-1);
}

/* Opens File, and if open fails exits with printing errno with perror().
 * Takes as parameters pointer to file name string and pointer to open modes
 *  string.
 * Returns pointer to FILE struct.
 */

FILE *open_file(char *path, char *modes)
{
  FILE *filep;

  filep = fopen(path, modes);
  if (filep == NULL) {
    perror(path);
    exit(-1);
  }

  return filep;
}

